/*
S-Monitor  a very simpel serial/USB monitor
Copyright (C) 2017  Christoph Mayer <feanors_fluch@web.de>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.
If not, see <https://www.gnu.org/licenses/gpl-3.0.txt>.
*/

//S-Monitor
//config.c

#include "config.h"

int newConfig(){
  int returnValue=0;
  configuration new;
  printf("Serial or USB port (s/u): ");
  scanf("%c",&new.port_type);
  printf("Portnumber: ");
  scanf("%hi",&new.port_no);
  printf("Baudrate: ");
  scanf("%i",&new.baudrate);
  printf("Databits: ");
  _flu;scanf("%c",&new.databits);
  printf("Parity (N/O/E): ");
  _flu;scanf("%c",&new.parity);
  printf("Stopbits: ");
  _flu;scanf("%c",&new.stopbits);
  printf("Delay [ms]: ");
  scanf("%i",&new.delay);
  returnValue=saveConfig(&new);
  if(returnValue>0) exit(returnValue);
  printf("\nConfiguration file saved successfully.\n\n");
  return(0);
}

int saveConfig(configuration *toSave){
  char *home,*configFile;
  FILE *FPconfigFile;
  home=getenv("HOME");
  configFile=malloc(strlen(home)+strlen("/.config/smon.conf")+1);
  if(configFile==NULL){
    fprintf(stderr,"\nMemory Error\n");
    fprintf(stderr,"ERROR %i: %s\n\n",errno,strerror(errno));
    return 23;
  }
  strcpy(configFile,home);
  strcat(configFile,"/.config");
  if(access(configFile,F_OK)!=0){
    if(mkdir(configFile,S_IRWXU)!=0){
      fprintf(stderr,"\nConfiguration file could not be created.\n");
      fprintf(stderr,"ERROR %i: %s\n\n",errno,strerror(errno));
      if(configFile!=NULL) free(configFile);
      return(1);
    }
  }
  strcat(configFile,"/smon.conf");
  FPconfigFile=fopen(configFile,"w");
  if(FPconfigFile==NULL){
    fprintf(stderr,"\nConfiguration file could not be created.\n");
    fprintf(stderr,"ERROR %i: %s\n\n",errno,strerror(errno));
    if(configFile!=NULL) free(configFile);
    return(1);
  }
  fputs("[port_settings]\n",FPconfigFile);
  fprintf(FPconfigFile,"port_type=%c\n",toSave->port_type);
  fprintf(FPconfigFile,"port_no=%i\n",toSave->port_no);
  fprintf(FPconfigFile,"baudrate=%i\n",toSave->baudrate);
  fprintf(FPconfigFile,"databits=%c\n",toSave->databits);
  fprintf(FPconfigFile,"parity=%c\n",toSave->parity);
  fprintf(FPconfigFile,"stopbits=%c\n",toSave->stopbits);
  fputs("\n[other_settings]\n",FPconfigFile);
  fprintf(FPconfigFile,"delay=%i\n",toSave->delay);
  if(fclose(FPconfigFile)==EOF){
    fprintf(stderr,"\nConfiguration file could not be saved.\n");
    fprintf(stderr,"ERROR %i: %s\n\n",errno,strerror(errno));
    if(configFile!=NULL) free(configFile);
    return(2);
  }
  if(configFile!=NULL) free(configFile);
  return 0;
}

int readConfig(configuration *toRead){
  char *home,*configFile,*buffer,*hit;
  long long int filesize;
  FILE *FPconfigFile;
  home=getenv("HOME");
  configFile=malloc(strlen(home)+strlen("/.config/smon.conf")+1);
  if(configFile==NULL){
    fprintf(stderr,"\nMemory Error\n");
    fprintf(stderr,"ERROR %i: %s\n\n",errno,strerror(errno));
    return 23;
  }
  strcpy(configFile,home);
  strcat(configFile,"/.config/smon.conf");
  FPconfigFile=fopen(configFile,"r");
  if(FPconfigFile==NULL){
    fprintf(stderr,"\nCould not open configuration File.\n");
    fprintf(stderr,"ERROR %i: %s\n\n",errno,strerror(errno));
    if(configFile!=NULL) free(configFile);
    return(3);
  }
  filesize=dateiGroesse(FPconfigFile);
  buffer=malloc(filesize+1);
  if(buffer==NULL){
    fprintf(stderr,"\nMemory Error\n");
    fprintf(stderr,"ERROR %i: %s\n\n",errno,strerror(errno));
    if(configFile!=NULL) free(configFile);
    return 23;
  }
  fread(buffer,1,filesize,FPconfigFile);
  hit=strstr(buffer,"port_type=");
  sscanf(hit,"port_type=%c\n",&toRead->port_type);
  if(toRead->port_type!='s' && toRead->port_type!='u'){
    fprintf(stderr,"\nInvalid port type in configuration File.\n\n");
    if(configFile!=NULL) free(configFile);
    if(buffer!=NULL) free(buffer);
    return(4);
  }
  hit=strstr(buffer,"port_no=");
  sscanf(hit,"port_no=%hi\n",&toRead->port_no);
  hit=strstr(buffer,"baudrate=");
  sscanf(hit,"baudrate=%i\n",&toRead->baudrate);
  hit=strstr(buffer,"databits=");
  sscanf(hit,"databits=%c\n",&toRead->databits);
  hit=strstr(buffer,"parity=");
  sscanf(hit,"parity=%c\n",&toRead->parity);
  hit=strstr(buffer,"stopbits=");
  sscanf(hit,"stopbits=%c\n",&toRead->stopbits);
  hit=strstr(buffer,"delay=");
  sscanf(hit,"delay=%i\n",&toRead->delay);
  if(buffer!=NULL) free(buffer);
  fclose(FPconfigFile);
  if(configFile!=NULL) free(configFile);
  return 0;
}
