/*
S-Monitor  a very simpel serial/USB monitor
Copyright (C) 2017  Christoph Mayer <feanors_fluch@web.de>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.
If not, see <https://www.gnu.org/licenses/gpl-3.0.txt>.
*/

//S-Monitor
//umon.c

#define _version "0.0.1"

#include <curses.h>

#include "meinzeug.h"
#include "rs232.h"
#include "config.h"

int monitor();
void help();
void about();

int main(int argc,char *argv[]){
  char option=0;
  int returnValue=0;
  if(argc==1) returnValue=monitor();
  else{
    if(strcmp(argv[1],"-c")==0) option='c';
    if(strcmp(argv[1],"-h")==0 || strcmp(argv[1],"--help")==0) option='h';
    if(strcmp(argv[1],"-a")==0) option='a';
    switch(option){
      case 'c': returnValue=newConfig();break;
      case 'h': help();break;
      case 'a': about();break;
    }
  }
  return(returnValue);
}

int monitor(){
  char mode[4],buffer[81];
  int openPort,bytesRead,isError=0;
  configuration current;
  isError=readConfig(&current);
  if(isError>0) return isError;
  if(current.port_type=='s') openPort=current.port_no;
  else openPort=16+current.port_no;
  mode[0]=current.databits;
  mode[1]=current.parity;
  mode[2]=current.stopbits;
  mode[3]=0;
  if(RS232_OpenComport(openPort,current.baudrate,mode)){
    fprintf(stderr,"\nSerial port could not be opened.\n\n");
    return(5);
  }
  initscr();
  cbreak();
  noecho();
  scrollok(stdscr,TRUE);
  nodelay(stdscr,TRUE);
  while(getch()==ERR){
    usleep(current.delay*1000);
    bytesRead=RS232_PollComport(openPort,(unsigned char *)buffer,80);
    if(bytesRead==0) printw("no data...\n");
    else{
      buffer[bytesRead]=0;
      printw("%s\n",buffer);
    }
    refresh();
  }
  RS232_CloseComport(openPort);
  endwin();
  return(0);
}

void help(){
  putchar(0xA);
  puts("SYNOPSIS");
  puts("\tsmon [options]");
  puts("DESCRIPTION");
  puts("\tS-Monitor is a very simple serial/USB monitor.");
  puts("\tFocus is on quick and easy use rather than on \"many Features\".");
  puts("OPTIONS");
  puts("\t-c\tcreate configuration file");
  puts("\t-a\tabout S-Monitor");
  puts("\t--help\tthis quickhelp");
  puts("\t-h\talias for --help");
  putchar(0xA);
}

void about(){
  putchar(0xA);
  puts("S-Monitor  "_version);
  puts("Copyright (C) 2017  Christoph Mayer <feanors_fluch@web.de>");
  putchar(0xA);
  puts("This program comes with ABSOLUTELY NO WARRANTY!");
  puts("This is free software, and you are welcome to redistribute it");
  puts("under certain conditions.");
  puts("See <https://www.gnu.org/licenses/gpl-3.0.txt> for details.");
  putchar(0xa);
}
